import { ObservedCall } from './ObservedCall';
import { ObservedCertificate } from './ObservedCertificate';
import { ObservedClient } from './ObservedClient';
import { ObservedCodec } from './ObservedCodec';
import { ObservedDataChannel } from './ObservedDataChannel';
import { ObservedIceCandidate } from './ObservedIceCandidate';
import { ObservedIceCandidatePair } from './ObservedIceCandidatePair';
import { ObservedIceTransport } from './ObservedIceTransport';
import { ObservedInboundRtp } from './ObservedInboundRtp';
import { ObservedInboundTrack } from './ObservedInboundTrack';
import { ObservedMediaPlayout } from './ObservedMediaPlayout';
import { ObservedMediaSource } from './ObservedMediaSource';
import { ObservedOutboundRtp } from './ObservedOutboundRtp';
import { ObservedOutboundTrack } from './ObservedOutboundTrack';
import { ObservedPeerConnection } from './ObservedPeerConnection';
import { ClientEvent, ClientIssue, ClientMetaData, ClientSample, ExtensionStat } from './schema/ClientSample';
export declare class ObservedCallEventMonitor<Context> {
    readonly call: ObservedCall;
    readonly context: Context;
    constructor(call: ObservedCall, context: Context);
    onCallClosed?: (call: ObservedCall, ctx: Context) => void;
    onCallEmpty?: (call: ObservedCall, ctx: Context) => void;
    onCallNotEmpty?: (call: ObservedCall, ctx: Context) => void;
    onCallUpdated?: (call: ObservedCall, ctx: Context) => void;
    onClientAdded?: (client: ObservedClient, ctx: Context) => void;
    onClientClosed?: (client: ObservedClient, ctx: Context) => void;
    onClientRejoined?: (client: ObservedClient, ctx: Context) => void;
    onClientIssue?: (observedClent: ObservedClient, issue: ClientIssue, ctx: Context) => void;
    onClientMetadata?: (observedClient: ObservedClient, metadata: ClientMetaData, ctx: Context) => void;
    onClientExtensionStats?: (observedClient: ObservedClient, extensionStats: ExtensionStat, ctx: Context) => void;
    onClientJoined?: (client: ObservedClient, ctx: Context) => void;
    onClientLeft?: (client: ObservedClient, ctx: Context) => void;
    onClientUserMediaError?: (observedClient: ObservedClient, error: string, ctx: Context) => void;
    onClientUsingTurn?: (client: ObservedClient, usingTurn: boolean, ctx: Context) => void;
    onClientUpdated?: (client: ObservedClient, sample: ClientSample, ctx: Context) => void;
    onClientEvent?: (client: ObservedClient, event: ClientEvent, ctx: Context) => void;
    onPeerConnectionAdded?: (peerConnection: ObservedPeerConnection, ctx: Context) => void;
    onPeerConnectionRemoved?: (peerConnection: ObservedPeerConnection, ctx: Context) => void;
    onSelectedCandidatePairChanged?: (peerConnection: ObservedPeerConnection, ctx: Context) => void;
    onIceGatheringStateChange?: (peerConnection: ObservedPeerConnection, ctx: Context) => void;
    onIceConnectionStateChange?: (peerConnection: ObservedPeerConnection, ctx: Context) => void;
    onConnectionStateChange?: (peerConnection: ObservedPeerConnection, ctx: Context) => void;
    onCertificateAdded?: (certificate: ObservedCertificate, ctx: Context) => void;
    onCertificateRemoved?: (certificate: ObservedCertificate, ctx: Context) => void;
    onInboundTrackAdded?: (inboundTrack: ObservedInboundTrack, ctx: Context) => void;
    onInboundTrackRemoved?: (inboundTrack: ObservedInboundTrack, ctx: Context) => void;
    onOutboundTrackAdded?: (outboundTrack: ObservedOutboundTrack, ctx: Context) => void;
    onOutboundTrackRemoved?: (outboundTrack: ObservedOutboundTrack, ctx: Context) => void;
    onInboundRtpAdded?: (inboundRtp: ObservedInboundRtp, ctx: Context) => void;
    onInboundRtpRemoved?: (inboundRtp: ObservedInboundRtp, ctx: Context) => void;
    onOutboundRtpAdded?: (outboundRtp: ObservedOutboundRtp, ctx: Context) => void;
    onOutboundRtpRemoved?: (outboundRtp: ObservedOutboundRtp, ctx: Context) => void;
    onDataChannelAdded?: (dataChannel: ObservedDataChannel, ctx: Context) => void;
    onDataChannelRemoved?: (dataChannel: ObservedDataChannel, ctx: Context) => void;
    onAddedIceTransport?: (iceTransport: ObservedIceTransport, ctx: Context) => void;
    onRemovedIceTransport?: (iceTransport: ObservedIceTransport, ctx: Context) => void;
    onIceCandidateAdded?: (iceCandidate: ObservedIceCandidate, ctx: Context) => void;
    onIceCandidateRemoved?: (iceCandidate: ObservedIceCandidate, ctx: Context) => void;
    onAddedIceCandidatePair?: (candidatePair: ObservedIceCandidatePair, ctx: Context) => void;
    onRemovedIceCandidatePair?: (candidatePair: ObservedIceCandidatePair, ctx: Context) => void;
    onAddedMediaCodec?: (codec: ObservedCodec, ctx: Context) => void;
    onRemovedMediaCodec?: (codec: ObservedCodec, ctx: Context) => void;
    onAddedMediaPlayout?: (mediaPlayout: ObservedMediaPlayout, ctx: Context) => void;
    onRemovedMediaPlayout?: (mediaPlayout: ObservedMediaPlayout, ctx: Context) => void;
    onMediaSourceAdded?: (mediaSource: ObservedMediaSource, ctx: Context) => void;
    onMediaSourceRemoved?: (mediaSource: ObservedMediaSource, ctx: Context) => void;
    private _onCallAdded;
    private _onClientAdded;
    private _onClientRejoined;
    private _onPeerConnconnectionAdded;
    private _onPeerConnectionRemoved;
    private _onCertificateAdded;
    private _onCertificateRemoved;
    private _onInboundTrackAdded;
    private _onInboundTrackRemoved;
    private _onOutboundTrackAdded;
    private _onOutboundTrackRemoved;
    private _onInboundRtpAdded;
    private _onInboundRtpRemoved;
    private _onOutboundRtpAdded;
    private _onOutboundRtpRemoved;
    private _onDataChannelAdded;
    private _onDataChannelRemoved;
    private _onAddedIceTransport;
    private _onRemovedIceTransport;
    private _onIceCandidateAdded;
    private _onIceCandidateRemoved;
    private _onAddedIceCandidatePair;
    private _onRemovedIceCandidatePair;
    private _onAddedMediaCodec;
    private _onRemovedMediaCodec;
    private _onAddedMediaPlayout;
    private _onRemovedMediaPlayout;
    private _onMediaSourceAdded;
    private _onMediaSourceRemoved;
    private _onClientIssue;
    private _onClientMetadata;
    private _onClientExtensionStats;
    private _onClientJoined;
    private _onClientLeft;
    private _onUserMediaError;
    private _onUsingTurn;
}
//# sourceMappingURL=ObservedCallEventMonitor.d.ts.map