/// <reference types="node" />
import { EventEmitter } from 'stream';
import { ObservedPeerConnection } from './ObservedPeerConnection';
import { ObservedTurnServer } from './ObservedTurnServer';
export type ObservedTURNEventMap = {
    'update': [ObservedTURN];
    'close': [];
};
export declare interface ObservedTURN {
    on<U extends keyof ObservedTURNEventMap>(event: U, listener: (...args: ObservedTURNEventMap[U]) => void): this;
    off<U extends keyof ObservedTURNEventMap>(event: U, listener: (...args: ObservedTURNEventMap[U]) => void): this;
    once<U extends keyof ObservedTURNEventMap>(event: U, listener: (...args: ObservedTURNEventMap[U]) => void): this;
    emit<U extends keyof ObservedTURNEventMap>(event: U, ...args: ObservedTURNEventMap[U]): boolean;
}
export declare class ObservedTURN extends EventEmitter {
    totalBytesSent: number;
    totalBytesReceived: number;
    totalPacketsSent: number;
    totalPacketsReceived: number;
    packetsSentPerSecond: number;
    packetsReceivedPerSecond: number;
    outboundBitrate: number;
    inboundBitrate: number;
    numberOfClients: number;
    readonly servers: Map<string, ObservedTurnServer>;
    constructor();
    update(): void;
    addPeerConnection(peerConnection: ObservedPeerConnection): ObservedTurnServer | undefined;
    removePeerConnection(peerConnection: ObservedPeerConnection): void;
}
//# sourceMappingURL=ObservedTURN.d.ts.map