"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Detectors = void 0;
const logger_1 = require("../common/logger");
const logger = (0, logger_1.createLogger)('Detectors');
class Detectors {
    _detectors;
    constructor(...detectors) {
        this._detectors = detectors;
    }
    add(detector) {
        this._detectors.push(detector);
    }
    remove(detector) {
        this._detectors = this._detectors.filter((d) => d !== detector);
    }
    get listOfNames() {
        return this._detectors.map((d) => d.name);
    }
    update() {
        for (const detector of this._detectors) {
            try {
                detector.update();
            }
            catch (err) {
                logger.warn(`Error updating detector ${detector?.constructor?.name}`, err);
            }
        }
    }
    clear() {
        this._detectors = [];
    }
}
exports.Detectors = Detectors;
